from fuo_kuwo.utils import unsigned_right_shift


class KwDecrypt:
    CONST_1 = b'kwks&@69'

    @staticmethod
    def aba(b_arr: bytes, i: int):
        i7 = int(((i * 4) + 2) / 3)
        c_arr = bytes()
        i8 = 0
        i9 = 0
        while i8 < i:
            i10 = i8 + 1
            b4 = b_arr[i8]
            if i10 < i:
                i2 = i10 + 1
                b2 = b_arr[i10] & 255
            else:
                i2 = i10
                b2 = 0
            if i2 < i:
                i3 = i2 + 1
                b3 = b_arr[i2] & 255
            else:
                i3 = i2
                b3 = 0
            i11 = unsigned_right_shift(b4, 2)
            i12 = ((b4 & 3) << 4) | unsigned_right_shift(b2, 4)




import logging
import time
from typing import List

from feeluown.excs import ProviderIOError
from feeluown.library import (
    AbstractProvider, ProviderV2, ModelType, BriefVideoModel, LyricModel,
    SupportsVideoMultiQuality, SupportsSongMultiQuality,
    SupportsAlbumGet, SupportsAlbumSongsReader,
    SupportsArtistGet, SupportsArtistAlbumsReader, SupportsArtistSongsReader,
    SupportsPlaylistGet, SupportsPlaylistSongsReader,
    SimpleSearchResult, SearchType,
)
from feeluown.library.model_protocol import BriefArtistProtocol, BriefSongProtocol, BriefVideoProtocol, BriefAlbumProtocol
from feeluown.media import Media, MediaType, Quality
from feeluown.utils.reader import SequentialReader

from .schemas import (
    KuwoSongSchema, KuwoAlbumSchema, KuwoArtistSchema, KuwoPlaylistSchema,
    KuwoUserPlaylistSchema,
)
from .utils import parse_lyrics
from . import __identifier__, __alias__
from .api import KuwoApi

logger = logging.getLogger(__name__)
Audio = Quality.Audio
Video = Quality.Video
SOURCE = __identifier__


class KuwoProvider(
    AbstractProvider, ProviderV2,
    SupportsVideoMultiQuality,
    SupportsSongMultiQuality,
    SupportsAlbumGet, SupportsAlbumSongsReader,
    SupportsArtistGet, SupportsArtistAlbumsReader, SupportsArtistSongsReader,
    SupportsPlaylistGet, SupportsPlaylistSongsReader,
):
    api: KuwoApi

    class meta:
        identifier: str = __identifier__
        name: str = __alias__

    def __init__(self):
        super().__init__()
        self.api = KuwoApi()
        self._user = None

    @property
    def user(self):
        return self._user

    @user.setter
    def user(self, user):
        self._user = user

    @property
    def name(self) -> str:
        return __alias__

    @property
    def identifier(self) -> str:
        return __identifier__

    def use_model_v2(self, model_type: ModelType) -> bool:
        return model_type in (ModelType.song, ModelType.user)

    """
    Requires: current user is not empty.
    """
    def current_user_playlists(self):
        data = self.api.get_user_playlists()
        data_playlists = data.get('plist', [])
        return [_deserialize(data_playlist, KuwoUserPlaylistSchema)
                for data_playlist in data_playlists]

    def rec_list_daily_playlists(self):
        data_playlists = self.api.playlist_recommend(20, 1)
        return [_deserialize(data_playlist, KuwoPlaylistSchema)
                for data_playlist in data_playlists.get('data', {}).get('list', [])]

    def song_get(self, identifier):
        data = self.api.get_song_detail(identifier)
        return _deserialize(data.get('data'), KuwoSongSchema)

    def song_list_quality(self, song: BriefSongProtocol) -> List[Audio]:
        has_lossless = self._model_cache_get_or_fetch(song, 'lossless')
        quality_list = [Quality.Audio.hq, Quality.Audio.lq]
        if has_lossless:
            quality_list.append(Quality.Audio.shq)
        return quality_list

    def song_get_media(self, song: BriefSongProtocol, quality: Audio):
        quality = quality.value
        if quality == 'lq':
            js = self.api.get_song_url(song.identifier)
            data = _get_data_or_raise(js)
            url = data.get('url', '')
            if not url:
                logger.warning("no song url for 'lq' quality")
                return None
            return Media(url,
                         format=KuwoApi.FORMATS_BRS[quality],
                         bitrate=KuwoApi.FORMATS_RATES[quality] // 1000)

        cache_key = f'media_{quality}'
        media, exists = song.cache_get(cache_key)
        if exists:
            return media

        # Response example::
        #   format=ape
        #   bitrate=1000
        #   url=http://sq.sycdn.kuwo.cn/xx/yy/zz.ape
        #   sig=1111111111111
        text = self.api.get_song_url_mobi(song.identifier, quality)
        media_data = {}
        for line in text.split():
            key, value = line.split('=', 1)
            media_data[key] = value
        # Check field value before use it since I'm not sure if the field always exists.
        if not media_data.get('url'):
            return None

        bitrate = int(media_data.get('bitrate', 0))
        if bitrate == 0:
            bitrate = KuwoApi.FORMATS_RATES[quality] // 1000
        media = Media(media_data['url'],
                      format=KuwoApi.FORMATS_BRS[quality],
                      bitrate=bitrate)
        song.cache_set(cache_key, media, ttl=int(time.time()) + 60 * 10)
        return media

    def song_get_lyric(self, song):
        data = self.api.get_song_lyrics(song.identifier)
        lyrics: list = data.get('data', {}).get('lrclist', [])
        return LyricModel(
            source=__identifier__,
            identifier=song.identifier,
            content=parse_lyrics(lyrics) or '',
        )

    def song_get_mv(self, song):
        hasmv = self._model_cache_get_or_fetch(song, 'hasmv')
        if not hasmv:
            return None
        return BriefVideoModel(
            source=__identifier__,
            identifier=f'mv_{self.identifier}',
            title=song.title,
        )

    def video_list_quality(self, _: BriefVideoProtocol) -> List[Video]:
        return [Video.sd]

    def video_get_media(self, video, _):
        identifier = video.identifier
        assert identifier.startswith('mv_')
        song_id = identifier[3:]
        js = self.api.get_song_mv(song_id)
        data = _get_data_or_raise(js)
        url = data.get('url', '')
        if not url:
            # TODO: remove this logging.
            logger.warning("song has no valid mv url, but attr 'hasmv' is true")
            return None
        return Media(url, MediaType.video)

    def album_get(self, identifier):
        data_album = self.api.get_album_info(identifier)
        return _deserialize(data_album['data'], KuwoAlbumSchema)

    def album_create_songs_rd(self, album: BriefAlbumProtocol):
        u_album = self.album_get(album.identifier)
        return u_album.songs

    def artist_get(self, identifier):
        data = self.api.get_artist_info(identifier)
        return _deserialize(data['data'], KuwoArtistSchema)

    def artist_create_songs_rd(self, artist: BriefArtistProtocol):
        return create_g(self.api.get_artist_songs, artist.identifier, KuwoSongSchema)

    def artist_create_albums_rd(self, artist: BriefArtistProtocol):
        return create_g(self.api.get_artist_albums,
                        artist.identifier,
                        KuwoAlbumSchema,
                        'albumList')

    def playlist_get(self, identifier):
        data_album = self.api.get_playlist_info(identifier)
        return _deserialize(data_album['data'], KuwoPlaylistSchema)

    def playlist_create_songs_rd(self, playlist):
        return create_g(self.api.get_playlist_info,
                        playlist.identifier,
                        KuwoSongSchema,
                        list_key='musicList')

    def search(self, keyword: str, **kwargs) -> SimpleSearchResult:
        type_ = SearchType.parse(kwargs['type_'])
        if type_ == SearchType.so:
            return search_song(keyword)
        if type_ == SearchType.al:
            return search_album(keyword)
        if type_ == SearchType.ar:
            return search_artist(keyword)
        if type_ == SearchType.pl:
            return search_playlist(keyword)
        return None


def create_g(func, identifier, schema, list_key='list'):
    data = func(identifier, page=1).get('data')
    total = int(data['total'])

    def g():
        nonlocal data
        if data is None:
            yield from ()
        else:
            page = 1
            while data[list_key]:
                obj_data_list = data[list_key]
                for obj_data in obj_data_list:
                    obj = _deserialize(obj_data, schema, gotten=True)
                    yield obj
                page += 1
                data = func(identifier, page=page).get('data', {})

    return SequentialReader(g(), total)


def _deserialize(data, schema_class, gotten=True):
    """ deserialize schema data to model

    :param data: data to be deserialize
    :param schema_class: schema class
    :param gotten:
    :return:
    """
    schema = schema_class()
    obj = schema.load(data)
    return obj


def _get_data_or_raise(js):
    """Get data from response json"""
    data = js.get('data')
    if not data:
        raise ProviderIOError(f"resp code is {js.get('code', -1000)}, expect 200")
    return data


def search_song(keyword: str):
    data_songs = provider.api.search(keyword)
    songs = []
    for data_song in data_songs.get('data', {}).get('list', []):
        song = _deserialize(data_song, KuwoSongSchema)
        songs.append(song)
    return SimpleSearchResult(q=keyword, songs=songs)


def search_album(keyword: str):
    data_albums = provider.api.search_album(keyword)
    albums = []
    for data_album in data_albums.get('data', {}).get('albumList', []):
        album = _deserialize(data_album, KuwoAlbumSchema)
        albums.append(album)
    return SimpleSearchResult(q=keyword, albums=albums)


def search_artist(keyword: str):
    data_artists = provider.api.search_artist(keyword)
    artists = []
    for data_artist in data_artists.get('data', {}).get('artistList', []):
        artist = _deserialize(data_artist, KuwoArtistSchema)
        artists.append(artist)
    return SimpleSearchResult(q=keyword, artists=artists)


def search_playlist(keyword: str):
    data_playlists = provider.api.search_playlist(keyword)
    playlists = []
    for data_playlist in data_playlists.get('data', {}).get('list', []):
        playlist = _deserialize(data_playlist, KuwoPlaylistSchema)
        playlists.append(playlist)
    return SimpleSearchResult(q=keyword, playlists=playlists)


provider = KuwoProvider()



import logging
import time
from pathlib import Path
from hashlib import md5, sha1

import requests
from requests.cookies import RequestsCookieJar
from fuo_kuwo.enc.DES import base64_encrypt
from fuo_kuwo.utils import digest_encrypt

logger = logging.getLogger(__name__)


class Singleton(type):
    """ singleton metaclass """
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class KuwoApi(object, metaclass=Singleton):
    """ kuwo music API class """
    API_BASE: str = 'http://www.kuwo.cn/api/www'
    HTTP_HOST: str = 'http://kuwo.cn'
    MOBI_HOST: str = 'http://mobi.kuwo.cn'
    M_HOST: str = 'http://m.kuwo.cn'
    SEARCH_HOST: str = 'http://search.kuwo.cn'
    LOGIN_HOST: str = 'http://ar.i.kuwo.cn/US_NEW/kuwo'
    PLAYLIST_BASE: str = 'http://nplserver.kuwo.cn/pl.svc'
    token: str
    cookie: RequestsCookieJar

    FORMATS_RATES = {
        'shq': 2000000,
        'hq': 320000,
        'sq': 192000,
        'lq': 128000
    }

    FORMATS_BRS = {
        'shq': '2000kflac',
        'hq': '320kmp3',
        'sq': '192kmp3',
        'lq': '128kmp3'
    }

    FORMATS = {
        'shq': 'AL',
        'hq': 'MP3H',
        'sq': 'MP3192',
        'lq': 'MP3128'
    }

    def __init__(self):
        """ class initializer """
        self.timeout = 30
        self.headers = {
            'Accept': '*/*',
            'Accept-Encoding': 'gzip,deflate',
            'Accept-Language': 'zh-CN,zh;q=0.8,gl;q=0.6,zh-TW;q=0.4',
            'Referer': 'http://kuwo.cn',
            'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/81.0.4044.138 Safari/537.36',
            'Host': 'kuwo.cn',
        }
        self.mobi_headers = {'User-Agent': 'okhttp/3.10.0'}
        random_str = '123456789'
        self.headers['csrf'] = random_str
        self.cookie = {'Hm_token': random_str}
        self.headers['Cross'] = md5(bytes(
            sha1(random_str.encode('utf-8')).hexdigest(), 'utf-8')).hexdigest()
        self._userid = ''
        self._sid = ''
        self._cookies = {}

    def set_cookies(self, cookies):
        if cookies:
            self._cookies = cookies
            self._userid, self._sid = self.get_user_from_cookies(cookies)
        else:
            self._cookies, self._userid, self._sid = {}, '', ''

    @staticmethod
    def get_user_from_cookies(cookies):
        userid = str(cookies.get('userid', ''))
        sid = str(cookies.get('sid', ''))
        if '_' in sid:
            sid = sid.split('_')[0]
        t3 = str(cookies.get('t3'))
        if t3.lower() in ['qq', 'weibo']:
            sid = str(cookies.get('websid', ''))
        return userid, sid

    def get_user_playlists(self):
        uri = KuwoApi.PLAYLIST_BASE
        params = {
            'op': 'getlistbyuid',
            'bigid': 1,
            'uid': self._userid,
        }
        with requests.Session() as session:
            response = session.get(uri, params=params, headers=self.mobi_headers)
            data = response.json()
            return data

    def search(self, keyword: str, limit=20, page=1) -> dict:
        """kuwo web search API

        :param keyword: keyword of song/artist/album
        :type keyword: str
        :param limit: numbers per page, defaults to 20
        :type limit: int
        :param page: current page, defaults to 1
        :type page: int
        :return: response data
        :rtype: dict
        """
        uri = KuwoApi.API_BASE + f'/search/searchMusicBykeyWord?key={keyword}&pn={page}&rn={limit}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def search_album(self, keyword: str, limit=20, page=1) -> dict:
        """kuwo search album API

        :param keyword: keyword of album
        :type keyword: str
        :param limit: numbers per page, defaults to 20
        :type limit: int
        :param page: current page
        :type page: int
        :return: response data
        :rtype: dict
        """
        uri = KuwoApi.API_BASE + f'/search/searchAlbumBykeyWord?key={keyword}&pn={page}&rn={limit}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def search_artist(self, keyword: str, limit=20, page=1) -> dict:
        """kuwo search artist list

        :param keyword: keyword of artist
        :type keyword: str
        :param limit: numbers per page, defaults to 20
        :type limit: int
        :param page: current page
        :type page: int
        :return: response data
        :rtype: dict
        """
        uri = KuwoApi.API_BASE + f'/search/searchArtistBykeyWord?key={keyword}&pn={page}&rn={limit}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def search_playlist(self, keyword: str, limit=20, page=1) -> dict:
        """kuwo search playlist API

        :param keyword: keyword of playlist
        :type keyword: str
        :param limit: numbers per page, defaults to 20
        :type limit: int
        :param page: current page
        :type page: int
        :return: response data
        :rtype: dict
        """
        uri = KuwoApi.API_BASE + f'/search/searchPlayListBykeyWord?key={keyword}&pn={page}&rn={limit}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def get_song_detail(self, rid: int) -> dict:
        """get song detail by rid

        :param rid: musicrid
        :type rid: int
        :return: song detail data
        :rtype: dict
        """
        uri = KuwoApi.API_BASE + f'/music/musicInfo?mid={rid}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def get_song_url(self, rid: int) -> dict:
        """get song url for web (128kmp3)

        :param rid: musicrid
        :type rid: int
        :return: song url data
        :rtype: dict
        """
        uri = KuwoApi.HTTP_HOST + f'/api/v1/www/music/playUrl?mid={rid}&type=music&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def get_song_url_mobi(self, rid: int, quality: str) -> str:
        """get song url for app (128/192/320kmp3, flac, app, ...)

        :param rid: musicrid
        :type rid: int
        :param quality: quality of song, choices are lq, sq, shq
        :type quality: str
        :return: song url, bitrate, format
        :rtype: str
        """
        if quality == 'shq':
            logger.debug(f'Querying lossless: {rid} ({quality})')
            formats = 'ape|flac|mp3|aac'
        else:
            logger.debug(f'Querying best mp3: {rid} ({quality})')
            formats = 'mp3|aac'
        payload = f'corp=kuwo&p2p=1&type=convert_url2&sig=0&format={formats}&rid={rid}'
        uri = KuwoApi.MOBI_HOST + '/mobi.s?f=kuwo&q=' + base64_encrypt(payload)
        with requests.Session() as session:
            response = session.get(uri, headers=self.mobi_headers)
            return response.text

    def get_album_info(self, aid: int, limit=20, page=1) -> dict:
        """ kuwo album info API

        :param aid: album id
        :type aid: int
        :param limit: song list numbers per page, defaults to 20
        :type limit: int
        :param page: song list current page
        :type page: int
        :return: response data
        :rtype: dict
        """
        uri = KuwoApi.API_BASE + f'/album/albumInfo?albumId={aid}&pn={page}&rn={limit}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def get_artist_info(self, aid: int, limit=20, page=1) -> dict:
        """ kuwo artist info API

        :param aid: album id
        :type aid: int
        :param limit: song list numbers per page, defaults to 20
        :type limit: int
        :param page: song list current page
        :type page: int
        :return: response data
        :rtype: dict
        """
        uri = KuwoApi.API_BASE + f'/artist/artist?artistid={aid}&pn={page}&rn={limit}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def get_playlist_info(self, pid: int, limit=20, page=1) -> dict:
        """ kuwo playlist info API

        :param pid: playlist id
        :type pid: int
        :param limit: song list numbers per page, defaults to 20
        :type limit: int
        :param page: song list current page
        :type page: int
        :return: response data
        :rtype: dict
        """
        uri = KuwoApi.API_BASE + f'/playlist/playListInfo?pid={pid}&pn={page}&rn={limit}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def get_artist_songs(self, aid: int, limit=20, page=1) -> dict:
        """ kuwo artist song list

        :param aid: album id
        :type aid: int
        :param limit: song list numbers per page, defaults to 20
        :type limit: int
        :param page: song list current page
        :type page: int
        :return: artist info response data
        :rtype: dict
        """
        uri = KuwoApi.API_BASE + f'/artist/artistMusic?artistid={aid}&pn={page}&rn={limit}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def get_artist_albums(self, aid: int, limit=20, page=1) -> dict:
        """ kuwo artist album list

        :param aid: album id
        :type aid: int
        :param limit: album list numbers per page, defaults to 20
        :type limit: int
        :param page: album list current page
        :type page: int
        :return: response data
        :rtype: dict
        """
        uri = KuwoApi.API_BASE + f'/artist/artistAlbum?artistid={aid}&pn={page}&rn={limit}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def get_song_mv(self, rid: int) -> dict:
        """ kuwo mv url API

        :param rid: musicrid
        :type rid: int
        :return: song mv url
        :rtype: dict
        """
        uri = KuwoApi.HTTP_HOST + f'/api/v1/www/music/playUrl?mid={rid}&type=mv&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            return response.json()

    def get_song_lyrics(self, rid: int) -> dict:
        """ kuwo song lyrics url

        :param rid: musicrid
        :type rid: int
        :return: song lyrics response data
        :rtype: dict
        """
        uri = KuwoApi.M_HOST + f'/newh5/singles/songinfoandlrc?musicId={rid}'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            return response.json()

    def get_mobile_verify_code(self, mobile: str, type_: int = 0):
        secret = digest_encrypt(digest_encrypt('imbadboy@!153').upper() + digest_encrypt(mobile + str(time.time()))
                                .upper()).upper()
        payload = f'mobile={mobile}&type={type_}&tm={str(time.time())}&secret={secret}'
        uri = KuwoApi.LOGIN_HOST + '/send_sms?f=ar&q=' + base64_encrypt(payload)
        with requests.Session() as session:
            response = session.post(uri, headers=self.mobi_headers)
            return response.text

    def radio_list(self) -> dict:
        """
        音乐电台
        :return:
        """
        uri = KuwoApi.API_BASE + f'/radio/index/radioList?&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def rank_index(self) -> dict:
        """
        榜单
        :return:
        """
        uri = KuwoApi.API_BASE + f'/bang/bang/bangMenu?&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def rank_music(self, bid: int, limit=20, page=1) -> dict:
        """
        榜单歌曲
        :param bid:
        :param limit:
        :param page:
        :return:
        """
        uri = KuwoApi.API_BASE + f'/bang/bang/musicList?bangId={bid}&pn={page}&rn={limit}&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def rank_top(self):
        """
        推荐榜单
        :return:
        """
        uri = KuwoApi.API_BASE + f'/bang/index/bangList?&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def playlist_recommend(self, limit=20, page=1) -> dict:
        """
        推荐歌单
        :param limit:
        :param page:
        :return:
        """
        uri = KuwoApi.API_BASE + f'/rcm/index/playlist?id=rec&pn={page}&rn={limit}&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def banner(self) -> dict:
        """
        首页轮播图
        :return:
        """
        uri = KuwoApi.API_BASE + f'/banner/index/bannerList?&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def comment(self, sid: int, type_: str, digest_: int, limit=20, page=1) -> dict:
        """
        评论
        :param sid:
        :param type_: get_rec_comment 热门评论 get_comment 最新评论
        :param digest_: 15 歌曲 2 排行榜 7 mv评论 8 歌单评论
        :param limit:
        :param page:
        :return:
        """
        uri = f'http://www.kuwo.cn/comment?type={type_}&f=web&page=${page}&rows=${limit}&digest=${digest_}&sid=${sid}' \
              f'&uid=&prod=newWeb&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def artists_recommend(self, category: int, limit=20, page=1) -> dict:
        """
        推荐歌手
        :param category: 11 華語  13 歐美  12 日韓 16 組合
        :param limit:
        :param page:
        :return:
        """
        uri = KuwoApi.API_BASE + f'/artist/artistInfo?category={category}&pn={page}&rn={limit}&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    def playlist_tags(self) -> dict:
        """
        歌单标签
        :return:
        """
        uri = KuwoApi.API_BASE + f'/playlist/getTagList?&httpsStatus=1'
        with requests.Session() as session:
            response = session.get(uri, cookies=self.cookie, headers=self.headers)
            data = response.json()
            return data

    @staticmethod
    def write_text_to_example(response: requests.Response, file_name: str):
        path = Path(__file__).parent.parent / 'examples' / f'{file_name}.json'
        with path.open('w', encoding='utf-8') as f:
            f.write(response.text)


if __name__ == '__main__':
    print(KuwoApi().playlist_tags())



import unicodedata
import html

from marshmallow import Schema, fields, post_load, EXCLUDE
from feeluown.library import (
    SongModel, BriefAlbumModel, BriefArtistModel,
    AlbumModel, ArtistModel, PlaylistModel, BriefPlaylistModel,
)

SOURCE = 'kuwo'


def normalize_str(s):
    # String returned from kuwo server may contain charactor
    # which is supposed to normalized.
    #
    # For example, there may be '&nbsp' in album name, which is
    # expected to normalized to whitespace.
    return unicodedata.normalize('NFKC', html.unescape(s))


def normalize_field(s):
    if s is None:
        return ''
    return normalize_str(s)


class BaseSchema(Schema):
    class Meta:
        unknown = EXCLUDE


Schema = BaseSchema


class KuwoSongSchema(Schema):
    identifier = fields.Int(data_key='rid', required=True)
    duration = fields.Int(data_key='duration', required=True)
    title = fields.Str(data_key='name', required=True)
    artist = fields.Str(data_key='artist', required=True)
    artistid = fields.Int(data_key='artistid', required=True)
    album = fields.Str(data_key='album', required=False)
    albumid = fields.Int(data_key='albumid', required=False)
    albumpic = fields.Str(data_key='albumpic', required=False, missing='')
    lossless = fields.Bool(data_key='hasLossless', required=False)
    hasmv = fields.Int(data_key='hasmv', required=False)

    @post_load
    def create_model(self, data, **kwargs):
        if data.get('artistid'):
            artists = [
                BriefArtistModel(
                    source=SOURCE,
                    identifier=data.get('artistid'),
                    name=normalize_field(data.get('artist'))
                )
            ]
        else:
            artists = []
        if data.get('albumid'):
            album = BriefAlbumModel(
                source=SOURCE,
                identifier=data.get('albumid'),
                name=normalize_field(data.get('album')),
            )
        else:
            album = None
        song = SongModel(
            source=SOURCE,
            identifier=str(data.get('identifier')),
            duration=data.get('duration') * 1000,
            title=normalize_field(data.get('title')),
            artists=artists,
            album=album,
            pic_url=data['albumpic'],
        )
        song.cache_set('lossless', data.get('lossless', False))  # bool
        song.cache_set('hasmv', bool(data.get('hasmv', 0)))  # int
        return song


class KuwoAlbumSchema(Schema):
    identifier = fields.Int(data_key='albumid', required=True)
    name = fields.Str(data_key='album', required=True)
    cover = fields.Str(data_key='pic', required=False, missing='')
    artist = fields.Str(data_key='artist', required=True)
    artistid = fields.Int(data_key='artistid', required=True)
    albuminfo = fields.Str(data_key='albuminfo', required=False)
    songs = fields.List(fields.Nested('KuwoSongSchema'),
                        data_key='musicList',
                        allow_none=True, required=False)
    song_count = fields.Int(data_key='total', required=False, missing=-1)
    released = fields.Str(data_key='releaseDate', required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return AlbumModel(
            source=SOURCE,
            identifier=data.get('identifier'),
            name=normalize_field(data.get('name')),
            artists=[BriefArtistModel(
                source=SOURCE,
                identifier=data.get('artistid'),
                name=normalize_field(data.get('artist')))],
            description=normalize_field(data.get('albuminfo', '')).replace('\n', '<br>'),
            cover=data.get('cover'),
            songs=data.get('songs') or [],
            song_count=data['song_count'],
            released=data['released'],
        )


class KuwoBriefArtistSchema(Schema):
    identifier = fields.Int(data_key='id', required=True)
    name = fields.Str(data_key='name', required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return BriefArtistModel(
            source=SOURCE,
            identifier=data.get['identifier'],
            name=normalize_field(data.get['name']),
        )


class KuwoArtistSchema(KuwoBriefArtistSchema):
    pic = fields.Str(data_key='pic', required=False, missing='')
    pic300 = fields.Str(data_key='pic300', required=False, missing='')
    desc = fields.Str(data_key='info', required=False, missing='')

    @post_load
    def create_model(self, data, **kwargs):
        return ArtistModel(
            source=SOURCE,
            identifier=data.get('identifier'),
            name=normalize_field(data.get('name')),
            pic_url=data.get('pic300'),
            aliases=[],
            hot_songs=[],
            description=normalize_field(data.get('desc')),
        )


class KuwoPlaylistSchema(Schema):
    identifier = fields.Int(data_key='id', required=True)
    cover = fields.Str(data_key='img', required=False, missing='')
    name = fields.Str(data_key='name', required=True)
    desc = fields.Str(data_key='info', required=False, missing='')

    @post_load
    def create_model(self, data, **kwargs):
        return PlaylistModel(
            source=SOURCE,
            identifier=data.get('identifier'),
            creator=None,
            name=normalize_field(data.get('name')),
            cover=data.get('cover'),
            description=data.get('desc'),
        )


class KuwoUserPlaylistSchema(Schema):
    identifier = fields.Int(data_key='id', required=True)
    cover = fields.Str(data_key='pic', required=False)
    name = fields.Str(data_key='title', required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return BriefPlaylistModel(
            source=SOURCE,
            identifier=data.get('identifier'),
            name=normalize_field(data.get('name')),
        )



# *-- coding: utf-8 --*
__alias__ = '酷我音乐'
__feeluown_version__ = '3.1'
__version__ = '0.1.1'
__desc__ = __alias__
__identifier__ = 'kuwo'

from feeluown.app import App

from .provider import provider


def enable(app: App):
    global ui_mgr
    app.library.register(provider)
    if app.mode & App.GuiMode:
        from .ui import ProviderUi

        provider_ui = ProviderUi(app)
        app.pvd_ui_mgr.register(provider_ui)


def disable(app: App):
    app.library.deregister(provider)
    if app.mode & App.GuiMode:
        app.providers.remove(provider.identifier)



import json
import os

from feeluown.consts import DATA_DIR
from feeluown.library import BriefUserModel
from feeluown.gui.widgets import CookiesLoginDialog
from feeluown.gui.widgets.login import InvalidCookies
from feeluown.utils import aio
from feeluown.gui.provider_ui import AbstractProviderUi

from . import __identifier__, __alias__
from .provider import provider

USER_INFO_FILE = DATA_DIR + '/kuwo_music_user_session.json'


class ProviderUi(AbstractProviderUi):
    def __init__(self, app):
        self._app = app

    @property
    def provider(self):
        return provider

    def get_colorful_svg(self) -> str:
        return os.path.join(os.path.dirname(__file__), 'assets', 'icon.svg')

    def login_or_go_home(self):
        if provider.user is None:
            self._dialog = dialog = LoginDialog()
            dialog.login_succeed.connect(lambda: aio.run_afn(self.load_user))
            dialog.show()
            dialog.autologin()
        else:
            aio.run_afn(self.load_user)

    async def load_user(self):
        self._app.ui.left_panel.playlists_con.show()
        self._app.pl_uimgr.clear()
        user_playlists = await aio.run_fn(provider.current_user_playlists)
        self._app.pl_uimgr.add(user_playlists)


class LoginDialog(CookiesLoginDialog):

    def setup_user(self, user):
        provider.user = user

    async def user_from_cookies(self, cookies):
        if not cookies:  # is None or empty
            raise InvalidCookies('empty cookies')

        userid, sid = provider.api.get_user_from_cookies(cookies)
        if not userid:
            raise InvalidCookies("can't extract user info from cookies")

        provider.api.set_cookies(cookies)
        # try to extract current user
        try:
            user = BriefUserModel(source=__identifier__, identifier=userid)
        except Exception:
            provider.api.set_cookies(None)
            raise InvalidCookies('get user info with cookies failed, expired cookies?')
        else:
            return user

    def load_user_cookies(self):
        if os.path.exists(USER_INFO_FILE):
            # if the file is broken, just raise error
            with open(USER_INFO_FILE) as f:
                return json.load(f).get('cookies', None)

    def dump_user_cookies(self, user, cookies):
        js = {
            'identifier': user.identifier,
            'name': user.name,
            'cookies': cookies
        }
        with open(USER_INFO_FILE, 'w') as f:
            json.dump(js, f, indent=2)



import ctypes
from html import unescape
from typing import Optional, Any


def int_overflow(val):
    maxint = 2147483647
    if not -maxint-1 <= val <= maxint:
        val = (val + (maxint + 1)) % (2 * (maxint + 1)) - maxint - 1
    return val


def unsigned_right_shift(n, i):
    if n < 0:
        n = ctypes.c_uint32(n).value
    if i < 0:
        return -int_overflow(n << abs(i))
    return int_overflow(n >> i)


def parse_lrc_time(time: str) -> str:
    """ parse float time string to mm:ss.ms

    :param time: float time string
    :type time: str
    :return: mm:ss.ms
    :rtype: str
    """
    time = float(time)
    minute = int(time // 60)
    seconds = int(time - minute * 60)
    ms = int((time - int(time)) * 1000)
    minutes_str = str(minute).zfill(2)
    seconds_str = str(seconds).zfill(2)
    ms_str = str(ms).ljust(3, '0')
    return f'{minutes_str}:{seconds_str}.{ms_str}'


def parse_lyrics(lyrics: Optional[list]) -> Optional[str]:
    """ parse lyrics to lrc file content

    :param lyrics: lyrics list
    :type lyrics: list, optional
    :return: lrc file content
    :rtype: str, optional
    """
    if not lyrics:
        return None
    content = []
    for line in lyrics:
        time = line.get('time')
        ll = line.get('lineLyric')
        if not time:
            continue
        content.append(f'[{parse_lrc_time(time)}]{unescape(ll)}')
    return "\n".join(content)


def digest_encrypt(s: str) -> str:
    from hashlib import md5
    return md5(s.encode('UTF-8')).digest().hex()
